/* gdm-init.c -- Initialize the Graphic Display Manager
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-hw.h>
#include <string.h>

/*! Control various modes of graphic display manager and LCD hardware.

    This operation allows to control the operating mode of the manager
    and of the hardware.  The \b mode is composed of flags:

    <dl>
    <dt>GDM_DISPLAY_ON
    <dd>can be passed to turn ON the display

    <dt>GDM_DISPLAY_OFF
    <dd>can be used to switch OFF the display

    <dt>GDM_BACKLIGHT_ON
    <dd>can be used to enable the backlight LED.
    </dl>

    @param dp   Display Manager
    @param mode Control flags  */
void
gdm_set_mode  (gdm_display* dp, int mode)
{
  _gdm_hw_set_side (dp, 0);
  _gdm_hw_set_mode (dp, mode);
  _gdm_hw_set_side (dp, 1);
  _gdm_hw_set_mode (dp, mode);
}

/*! Set the mode of graphic operations.

    The mode controls the logical operation used to perform the drawing
    operations.

    @param dp  Display Manager
    @param mode Logical operation to use  */
void
gdm_set_gc (gdm_display* dp, enum gdm_gc_mode mode)
{
  dp->plot_mode = mode;
  dp->or_mask = 0;
  dp->xor_mask = 0;
  dp->and_mask = 0;
  switch (mode)
    {
    case GDM_PLOT_OR:
      dp->or_mask = 0xff;
      break;

    case GDM_PLOT_XOR:
      dp->xor_mask = 0xff;
      break;

    case GDM_PLOT_AND:
      dp->and_mask = 0xff;
      break;
    }
}

/*! Initialize the graphic display manager.

    The graphic display manager must be initialized to setup the initial
    image and internal data structures used to represent the display.
    The graphical context is set to the \b GDM_PLOT_OR mode.  The LCD
    hardware is turned on.

    @param dp   Display manager data
    @see gdm_refresh, gdm_set_gc, gdm_set_mode, gdm_touch  */
void
gdm_initialize (gdm_display* dp)
{
  unsigned short i;
  gdm_line* l;

  memset (dp, 0, sizeof (dp[0]));

  /* Set the pointers to access a given line.  The 'lines' pointer table
     is used to provide a fast access and replaces a multiplication.
     Setup the line information so that refreshing can be efficient.  */
  l = &dp->image[0];
  for (i = 0; i < GDM_LCNT; i++, l++)
    {
      dp->lines[i] = l;
      l->y = i;

      /* Force a refresh of the line.  */
      l->touched = 0xff;
    }
  gdm_set_gc (dp, GDM_PLOT_OR);
  gdm_set_mode (dp, GDM_DISPLAY_ON);
}

