/* gdm-lines.c -- Line Drawing
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-raw.h>

/*! Draw a line between two points.

    Draw a line between @b x1, @b y1 and @b x2, @b y2.  The line is clipped
    to the display window and using the current clipping region.  The points
    of the line are drawn using the current graphical context (or, and, xor).

    @param dp   Display Manager
    @param x1   X coordinate of first point
    @param y1   Y coordinate of first point
    @param x2   X coordinate of second point
    @param y2   Y coordinate of second point
    @see gdm_draw_lines, gdm_refresh  */
void
gdm_draw_line (gdm_display* dp, short x1, short y1, short x2, short y2)
{
  short u;
  short v;
  short d1x, max, d1y, n, s, d2x, d2y, x, y, i;

  if (x2 < x1)
    {
      u = x2;
      x2 = x1;
      x1 = u;
      u = y2;
      y2 = y1;
      y1 = u;
    }

  u = x2 - x1;
  v = y2 - y1;
  x = x1;
  y = y1;
  if (u)
    {
      d1x = 1;
      max = u;
    }
  else
    {
      d1x = 0;
      max = 0;
    }
  if (v > 0)
    {
      d1y = 1;
      n = v;
    }
  else if (v < 0)
    {
      d1y = -1;
      n = -v;
    }
  else
    {
      d1y = 0;
      n = 0;
    }
  if (max < n)
    {
      /* swap */
      d2x = max;
      max = n;
      n = d2x;

      d2x = 0;
      d2y = d1y;
    }
  else
    {
      d2x = d1x;
      d2y = 0;
    }
  s = max >> 1;
  for (i = 0; i <= max; i++ )
    {
      gdm_draw_point (dp, x, y);
      s += n;
      if (s >= max)
        {
          s -= max;
          x += d1x;
          y += d1y;
        }
      else
        {
          x += d2x;
          y += d2y;
        }
    }
}

/*! Draw a continuous line, some segments or a polygon.

    Draw a continuous line, a set of segments or a polygon based on
    the list of points specified in @b pts.  The @b mode parameter
    controls the drawing mode:

    GDM_LINE     A continuous line is drawn between each point of the list.
    GDM_POLYGON  The continous line is closed to form a polygon
    GDM_SEGMENT  The list of points represents couples of points to
                 define one or several segments.  @b npoints / 2 segments
                 are drawn.

    @param dp   Display Manager
    @param pts  Table of points to display
    @param npoints Number of points in the table
    @param mode    Mode to display the line
    @see gdm_draw_line, gdm_refresh  */
void
gdm_draw_lines (gdm_display* dp, gdm_point* pts,
                unsigned short npoints, short mode)
{
  gdm_point* p = pts;

  while (npoints > 1)
    {
      gdm_draw_line (dp, p[0].x, p[0].y, p[1].x, p[1].y);
      if (mode == GDM_SEGMENT)
        {
          p += 2;
          npoints -= 2;
        }
      else
        {
          p += 1;
          npoints -= 1;
        }
    }

  /* For a polygon, add a line between the first and last point.  */
  if (mode == GDM_POLYGON)
    gdm_draw_line (dp, pts[0].x, pts[0].y, p[0].x, p[0].x);
}

