/* gdm-plot.c -- Graphic Display Manager Basic Plotter
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-raw.h>

const gdm_mask _gdm_bitmask[] = {
  0x01,
  0x02,
  0x04,
  0x08,
  0x10,
  0x20,
  0x40,
  0x80
};

const gdm_mask _gdm_ibitmask[] = {
  ~0x01,
  ~0x02,
  ~0x04,
  ~0x08,
  ~0x10,
  ~0x20,
  ~0x40,
  (gdm_mask) ~0x80
};

const gdm_mask _gdm_start_mask[] = {
  0xFF,
  0xFE,
  0xFC,
  0xF8,
  0xF0,
  0xE0,
  0xC0,
  0x80
};

const gdm_mask _gdm_end_mask[] = {
  0x01,
  0x03,
  0x07,
  0x0F,
  0x1F,
  0x3F,
  0x7F,
  0xFF
};

void _gdm_plot (gdm_display* dp, unsigned short x, unsigned short y)
{
  struct gdm_line* l = _gdm_get_raw_line (dp, y);

  _gdm_raw_plot (dp, &l->data[x], _gdm_bitmask[y & 7]);
  _gdm_touch_line (l, x);
}

/*! Draw a point on the graphic display.

    The point at @b x, @b y is plotted using the current color and
    graphic operation.  The point is clipped to the current display
    clipping region.  The point is plotted in the bitmap only and a display
    refresh is necessary to make the change visible.

    @param dp  Display manager
    @param x   X coordinate
    @param y   Y coordinate
    @see gdm_draw_points, gdm_refresh  */
void
gdm_draw_point (gdm_display* dp, short x, short y)
{
  gdm_clip_point_handler clip;

  /* Clip to the display size.  */
  if (x < 0 || x >= GDM_WIDTH)
    return;
  if (y < 0 || y >= GDM_HEIGHT)
    return;

  /* If a specific clip is defined, call it  */
  clip = dp->clip_point;
  if (clip && clip (dp, x, y) != 0)
    return;

  _gdm_plot (dp, x, y);
}

/*! Draw a list of points on the graphic display.

    The points specified in the table @b pts are plotted using the
    current color and graphic operation.  The points are clipped to
    the current display clipping region.  The point is plotted in the
    bitmap only and a display refresh is necessary to make the change visible.

    @param dp  Display manager
    @param pts Table of points to display
    @param npoints Number of points to print
    @see gdm_draw_point, gdm_refresh  */
void
gdm_draw_points (gdm_display* dp, gdm_point* pts,
                 unsigned short npoints)
{
  if (npoints != 0)
    {
      do {
        gdm_draw_point (dp, pts->x, pts->y);
        pts++;
      } while (--npoints != 0);
    }
}

