/* gdm-refresh.c -- Graphics Display Manager Refresh Module
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-raw.h>
#include <gdm/display-hw.h>

void
_gdm_do_refresh (gdm_display* dp, gdm_line* l,
                 unsigned short x_first, unsigned short x_last)
{
  unsigned char* p = &l->data[x_first];

  /* Update the left side of LCD  */
  if (x_first < 64)
    {
      unsigned char* q;

      _gdm_hw_set_side (dp, 1);

      /* Setup the LCD X register (Y for us) and keep track of it so that
         we reduce the number of LCD accesses.  */
      if (dp->hw0_cur_y != l->y)
        {
          _gdm_hw_set_x (dp, l->y);
          dp->hw0_cur_y = l->y;
        }

      /* Setup the LCD Y register (X for us) and write the data starting
         from it.  The LCD Y register is incremented after each write and
         therefore we don't update it after each data write.  We must stop
         when we reach the last position to update or move to the right
         side of LCD.  */
      _gdm_hw_set_y (dp, x_first);

      q = &l->data[x_last >= 64 ? 63 : x_last];
      while (p <= q)
        {
          _gdm_hw_set_data (dp, *p);
          p++;
        }
      x_first = 64;
    }

  /* Update the right side.  */
  if (x_last >= 64)
    {
      unsigned char* q;

      _gdm_hw_set_side (dp, 0);

      if (dp->hw1_cur_y != l->y)
        {
          _gdm_hw_set_x (dp, l->y);
          dp->hw1_cur_y = l->y;
        }
      _gdm_hw_set_y (dp, x_first - 64);

      q = &l->data[x_last];
      while (p <= q)
        {
          _gdm_hw_set_data (dp, *p);
          p++;
        }
    }
}

int
_gdm_refresh_line (gdm_display* dp, gdm_line* l, gdm_line* screen)
{
  unsigned short i;
  unsigned short x_first = 0xffff;
  unsigned short x_last = 0;
  gdm_mask chg = l->touched;

  for (i = 0; chg != 0; i++, chg >>= 1)
    {
      unsigned short x;

      if ((chg & 1) == 0)
        continue;

      x = i << 4;
      if (x < x_first)
        {
          x_first = x;
        }
      else if (x - x_last > 2)
        {
          _gdm_do_refresh (dp, l, x_first, x_last + 15);
          x_first = x;
          x_last = x;
        }

      if (x > x_last)
        x_last = x;
    }
  if (x_last >= x_first)
    _gdm_do_refresh (dp, l, x_first, x_last + 15);

  chg = l->touched;
  l->touched = 0;
  return chg;
}

/*! Refresh the graphic display.

    Update the graphic display by synchronizing the data image that we
    keep track in @b dp and sending the appropriate commands to the LCD
    controller.  The data image is scanned row by row and only the
    row parts that have changed are synchronized.  This ensures a minimal
    interaction to the LCD controller and thus speed up the drawing and
    refresh process.

    @param dp  Display Manager.
    @see gdm_touch  */
int
gdm_refresh (gdm_display* dp)
{
  unsigned short i;
  int changed = 0;

  for (i = 0; i < GDM_LCNT; i++)
    {
      changed |= _gdm_refresh_line (dp, dp->lines[i], 0);
    }
  return changed;
}

