/* sci.c -- SCI Controller
   Copyright 2000 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/interrupts.h>
#include <sys/ports.h>
#include <gel/event.h>
#include <gel/sci.h>
#include <stddef.h>


typedef struct sci_queue 
{
  unsigned char *sci_queue_rd;
  unsigned char *sci_queue_wr;
  unsigned char *sci_queue_first;
  unsigned char *sci_queue_last;

  unsigned char sci_queue_read_ok;
  unsigned char sci_queue_write_ok;

  unsigned char sci_working;
} sci_queue_t;

static sci_queue_t sci_recv_queue;
static sci_queue_t sci_send_queue;

int sci_drop_char;

struct event_def sci_receive;
struct event_def sci_send_done;

/* Insert a character at end of the queue.  If the queue is full,
   clear the write ok flag and return 1.  Otherwise, set the read ok
   flag and return 0.  */
static inline int
append_queue_char (sci_queue_t *queue, unsigned char ch)
{
  unsigned char *p;

  p = queue->sci_queue_wr;
  p++;
  if (p == queue->sci_queue_last)
    p = queue->sci_queue_first;
  
  if (p == queue->sci_queue_rd)
    {
      queue->sci_queue_write_ok = 0;
      return 1;
    }

  *p = ch;
  queue->sci_queue_wr = p;

  /* Mark queue as ready for reading since it has something in it.  */
  queue->sci_queue_read_ok = 1;
  return 0;
}

/* Pick the first character of the queue.  If the queue is empty,
   clear the read ok flag and return -1.  Otherwise, set the write
   ok flag and return the character (range 0..255).  */
static inline int
pick_queue_char (sci_queue_t *queue)
{
  unsigned char *p;
  unsigned short ch;
  
  p = queue->sci_queue_rd;
  if (p == queue->sci_queue_wr)
    {
      queue->sci_queue_read_ok = 0;
      return -1;
    }
  
  ch = (unsigned short) *p++;
  if (p == queue->sci_queue_last)
    p = queue->sci_queue_first;
  
  queue->sci_queue_rd = p;

  /* Mark queue as ready for writing since there is room
     for at least one char.  */
  queue->sci_queue_write_ok = 1;
  return ch;
}

static void
sci_send_char (void)
{
  sci_queue_t *queue;
  int ch;

  queue = &sci_send_queue;
  ch = pick_queue_char (queue);
  if (ch >= 0)
    {
      _io_ports[M6811_SCDR] = ch;
    }
  else
    {
      _io_ports[M6811_SCCR2] &= ~(M6811_TIE | M6811_TCIE);
      queue->sci_working = 0;
      event_post (&sci_send_done, 0);
    }
}

static void
sci_receive_char (unsigned char c)
{
  sci_queue_t *queue;
  
  queue = &sci_recv_queue;

  /* Post an event when we receive the first character of the queue.
     This allows a handler to be activated and take into account
     the characters in the read queue.  */
  if (queue->sci_queue_rd == queue->sci_queue_wr)
    event_post (&sci_receive, 0);

  if (append_queue_char (queue, c) != 0)
    {
      sci_drop_char++;
      return;
    }
}

void __attribute__((interrupt))
sci_interrupt (void)
{
  unsigned char flags;

  /* Need to read the SPSR first.  */
  flags = _io_ports[M6811_SCSR];

  /* Treat reception first to reduce buffer overrun problems.  */
  if (flags & M6811_RDRF)
    {
      sci_receive_char (_io_ports[M6811_SCDR]);
    }
  if (flags & M6811_TDRE)
    {
      sci_send_char ();
    }
}

void
sci_putc (unsigned char c)
{
  sci_queue_t *queue;

  queue = &sci_send_queue;
  while (1)
    {
      int result;
      unsigned short mask;
      
      mask = lock ();

      /* If the SCI is not active, send the first character.  */
      if (queue->sci_working == 0)
        {
          _io_ports[M6811_SCCR2] |= M6811_TIE;
          _io_ports[M6811_SCDR] = c;
          queue->sci_working = 1;
          restore (mask);
          break;
        }

      /* Otherwise, put the character in the queue.  It will be picked
         from the interrupt handler.  */
      result = append_queue_char (queue, c);
      restore (mask);
      if (result == 0)
	break;

      /* When the queue is full, wait for it to become ready for writing.  */
      event_loop (1, &queue->sci_queue_write_ok);
    }
}

void
sci_write (const unsigned char *data, size_t len)
{
  if (len)
    do
    {
      sci_putc (*data++);
    } while (--len != 0);
}

int
sci_read (unsigned char *data, size_t len, int mode)
{
  size_t l;
  unsigned short mask;
  
  for (l = 0; l != len; )
    {
      int ch;
      
      mask = lock ();
      ch = pick_queue_char (&sci_recv_queue);
      restore (mask);
      if (ch >= 0)
	{
	  *data++ = ch;
	  l++;
	  continue;
	}

      /* Wait for the queue to become ready for reading.  */
      event_loop (1, &sci_recv_queue.sci_queue_read_ok);
    }
  return l;
}

/**
 * Initialize the sci package.
 *
 */
void
sci_initialize (unsigned char baud, unsigned char *buffer, size_t len)
{
  sci_queue_t *q;

  len >>= 1;
  q = &sci_recv_queue;
  q->sci_queue_rd = buffer;
  q->sci_queue_wr = buffer;
  q->sci_queue_first = buffer;
  buffer += len;
  q->sci_queue_last = buffer;
  q->sci_queue_read_ok = 0;
  q->sci_queue_write_ok = 1;
  q->sci_working = 0;

  q = &sci_send_queue;
  q->sci_queue_rd = buffer;
  q->sci_queue_wr = buffer;
  q->sci_queue_first = buffer;
  buffer += len;
  q->sci_queue_last = buffer;
  q->sci_queue_read_ok = 0;
  q->sci_queue_write_ok = 1;
  q->sci_working = 0;

  /* Install the SCI interrupt handler.  */
  set_interrupt_handler (SCI_VECTOR, sci_interrupt);

  _io_ports[M6811_BAUD] = baud;

  /* Setup character format 1 start, 8-bits, 1 stop.  */
  _io_ports[M6811_SCCR1] = 0;

  /* Enable reciever and transmitter.  */
  _io_ports[M6811_SCCR2] = M6811_RIE | M6811_TE | M6811_RE;
}

