/* timer-test.c -- Test of timer and LCD
   Copyright 2000, 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/interrupts.h>
#include <gel/timer.h>
#include <sys/sio.h>
#include <stddef.h>
#include <benchs.h>
#include <test.h>

#ifdef USE_INTERRUPT_TABLE

/* Interrupt table used to connect our GEL interrupt handlers.

   Note: the `XXX_handler: foo' notation is a GNU extension which is
   used here to ensure correct association of the handler in the struct.
   This is why the order of handlers declared below does not follow
   the HC11 order.  */
struct interrupt_vectors __attribute__((section(".vectors"))) vectors = 
{
  res0_handler:           fatal_interrupt, /* res0 */
  res1_handler:           fatal_interrupt,
  res2_handler:           fatal_interrupt,
  res3_handler:           fatal_interrupt,
  res4_handler:           fatal_interrupt,
  res5_handler:           fatal_interrupt,
  res6_handler:           fatal_interrupt,
  res7_handler:           fatal_interrupt,
  res8_handler:           fatal_interrupt,
  res9_handler:           fatal_interrupt,
  res10_handler:          fatal_interrupt, /* res 10 */
  sci_handler:            fatal_interrupt, /* sci */
  spi_handler:            fatal_interrupt, /* spi */
  acc_overflow_handler:   fatal_interrupt, /* acc overflow */
  acc_input_handler:      fatal_interrupt,
  output5_handler:        fatal_interrupt, /* out compare 5 */
  output4_handler:        fatal_interrupt, /* out compare 4 */
  output3_handler:        fatal_interrupt, /* out compare 3 */
  output2_handler:        fatal_interrupt, /* out compare 2 */
  output1_handler:        fatal_interrupt, /* out compare 1 */
  capture3_handler:       fatal_interrupt, /* in capt 3 */
  capture2_handler:       fatal_interrupt, /* in capt 2 */
  capture1_handler:       fatal_interrupt, /* in capt 1 */
  irq_handler:            fatal_interrupt, /* IRQ */
  xirq_handler:           fatal_interrupt, /* XIRQ */
  swi_handler:            fatal_interrupt, /* swi */
  illegal_handler:        fatal_interrupt, /* illegal */
  cop_fail_handler:       fatal_interrupt,
  cop_clock_handler:      fatal_interrupt,

  /* What we really need.  */
  timer_overflow_handler: timer_overflow_interrupt,
  rtii_handler:           timer_interrupt,
  reset_handler:          _start
};

#endif

#define NR_TIMERS 64

#define TIMER_SUM_DELAY ((64L * 64L * 1000L) / 1000000L)

extern int rand (void);

/* Forward declarations.  */
void test_timer_handler (struct timer *t);
int test_timer_create (int reverse, timer_handler handler);
int test_timer_remove (void);
int test_timer_activate (void);
int test_time (long count);
void test_timer_accuracy (struct timer *t);
int test_timer_wait_all_done (void);
void timer_benchs (void);


struct timer timer_table[NR_TIMERS];
long timer_table_cnt[NR_TIMERS];
char timer_error = 0;
struct timeval time_table[NR_TIMERS];
long error_count;

static struct timeval *
test_get_timeval (struct timer *t)
{
  int pos;

  pos = (size_t) (t - &timer_table[0]);
  if (pos < 0 || pos >= NR_TIMERS)
    {
      test_error ("invalid timer object, out of bounds");
      return &time_table[0];
    }
  return &time_table[pos];
}

/* Test the timer accuracy by getting the time when the timer is
   fired and making sure that the difference with the time it was
   planned is less than a clock tick (4ms).  */
void
test_timer_accuracy (struct timer *t)
{
  struct timeval *tv;
  struct timeval wakeup_time;
  struct timeval dt;
  int pos;

  /* Get current time.  */
  timer_gettime (&wakeup_time);

  /* Get timer planned fired time.  */
  tv = test_get_timeval (t);
  if (tv == 0)
    return;

  pos = (size_t) (t - &timer_table[0]);
  /*printf ("Timer %ld fired\n", (long) pos);*/
  dt = wakeup_time;
  timevalsub (&dt, tv);

  if (dt.tv_sec > 2)
    {
      error_count++;
    }
  time_table[pos] = dt;
  /* Choose a timer randomly.  */
  pos = rand () % NR_TIMERS;
  if (pos < 0)
    pos = -pos;
  t = &timer_table[pos];

  /* Remove it if it is not fired.  */
  if (timer_is_active (t))
    {
      /* test_error ("Remove %ld\n", (long) pos); */
      timer_remove (t);
      time_table[pos].tv_sec = 0;
      time_table[pos].tv_usec = 0;
    }
}

void
test_timer_handler (struct timer *t)
{
  int id;

  if (t < &timer_table[0])
    timer_error = 1;
  
  id = t - &timer_table[0];
  if (id >= NR_TIMERS)
    timer_error = 1;
  else
    timer_table_cnt[id]++;
}

int
test_timer_create (int reverse, timer_handler handler)
{
  unsigned i;

  timer_error = 0;
  for (i = 0; i < NR_TIMERS; i++)
    {
      struct timeval tv;
      unsigned long timeout;

      if (reverse)
        timeout = NR_TIMERS * 12L - ((unsigned long) i) * 10L + 1L;
      else
        timeout = ((unsigned long) i + 1) * 10L + 1L;

      tv.tv_sec = timeout / 1000L;
      tv.tv_usec = (timeout % 1000L) * 1000L;
      timer_table_cnt[i] = 0;
      timer_create (&timer_table[i], timeout, handler, TIMER_UNMASK);
      timer_gettime (&time_table[i]);
      timevaladd (&time_table[i], &tv);
    }
  return timer_error;
}

int
test_timer_remove ()
{
  int i;

  for (i = 0; i < NR_TIMERS; i++)
    {
      timer_remove (&timer_table[i]);
    }
  return timer_error;
}

int
test_timer_activate ()
{
  int done;
  int i;
  struct timeval tv;
  struct timeval tstart;

  timer_gettime (&tstart);
  while (1)
    {
      done = 1;
      for (i = 0; i < NR_TIMERS; i++)
	{
	  if (timer_table_cnt[i] == 0)
	    {
	      done = 0;
	      break;
	    }
	}

      timer_gettime (&tv);
      timevalsub (&tv, &tstart);
      if (tv.tv_sec > TIMER_SUM_DELAY)
	return test_error ("timer handlers not called");

      if (done == 1)
	break;
    }
  return 0;
}

int
test_timer_wait_all_done ()
{
  int done;
  int i;
  struct timeval tstart;

  timer_gettime (&tstart);
  do
    {
      done = 0;
      for (i = 0; i < NR_TIMERS; i++)
	{
          int active;

	  lock ();
	  active = timer_is_active (&timer_table[i]);
	  unlock ();
	  if (active)
	    {
	      done = 1;
	      break;
	    }
	}
    }
  while (done == 1);
  
  for (i = 0; i < NR_TIMERS; i++)
    {
      struct timeval dt = time_table[i];
      
      if (dt.tv_sec == -1)
        {
          dt.tv_usec = 1000000 - dt.tv_usec;
          dt.tv_sec = 0;
        }
      if (dt.tv_sec != 0)
        {
          test_error ("timer %ld bad accuracy %ld secs",
                      (long) i, dt.tv_sec);
        }
      else if (dt.tv_usec > 8000 || dt.tv_usec < -8000)
        {
          test_error ("timer %ld bad accuracy %ld usecs",
                      (long) i, dt.tv_usec);
        }
    }
  return 0;
}

int
test_time (long count)
{
  struct timeval tv;
  struct timeval prev_time;
  struct timeval dt;
  
  timer_gettime (&prev_time);
  while (--count > 0)
    {
      timer_gettime (&tv);
      dt = tv;
      timevalsub (&dt, &prev_time);

      /* The time must always increase and the delta time
	 between two consecutive calls should not exceed 1 ms.  */
      if (dt.tv_sec != 0)
	return test_error ("timer_gettime seconds");
      if (dt.tv_usec < 0 || dt.tv_usec > 1000)
	return test_error ("timer_gettime microseconds");
      prev_time = tv;
    }
  return 0;
}

void
timer_benchs ()
{
  bench_t b;
  struct timeval tv;
  struct timeval d2;
  int i;

  bench_init (&b);

  bench_start (&b);
  timer_gettime (&tv);
  bench_stop (&b);
  bench_report (&b, "timer_gettime");

  bench_start (&b);
  timer_gettime (&d2);
  bench_stop (&b);
  bench_report (&b, "timer_gettime (2)");

  bench_start (&b);
  timevaladd (&d2, &tv);
  bench_stop (&b);
  bench_report (&b, "timevaladd");

  bench_start (&b);
  timevalsub (&d2, &tv);
  bench_stop (&b);
  bench_report (&b, "timevalsub");

  bench_start (&b);
  timer_create (&timer_table[0], 100000, test_timer_handler, TIMER_UNMASK);
  bench_stop (&b);
  bench_report (&b, "timer_create (1 timer)");

  for (i = 0; i < 9; i++)
    timer_create (&timer_table[i+1], i * 100, test_timer_handler,
                  TIMER_UNMASK);

  bench_start (&b);
  timer_create (&timer_table[11], 1000000, test_timer_handler, TIMER_UNMASK);
  bench_stop (&b);
  bench_report (&b, "timer_create (10 timers)");

  bench_start (&b);
  timer_remove (&timer_table[11]);
  bench_stop (&b);
  bench_report (&b, "timer_remove (last timer)");

  bench_start (&b);
  timer_remove (&timer_table[1]);
  bench_stop (&b);
  bench_report (&b, "timer_remove (first timer)");
}

int
main()
{
  int i;
  int result;
  int mode;
  
  serial_init ();
  timer_initialize ();
  test_depend ("serial");
  test_start ("timer-test");

  mode = 0;
  test_repeat = 2;
  for (i = 0; i < test_repeat; i++)
    {
      result = test_time (100);
      if (result != 0)
	break;

      result = test_timer_create (mode, test_timer_handler);
      if (result != 0)
	break;

      result = test_timer_activate ();
      if (result != 0)
	break;

      result = test_timer_remove ();
      if (result != 0)
	break;

      result = test_timer_create (mode, test_timer_handler);
      if (result != 0)
	break;

      result = test_timer_remove ();
      if (result != 0)
	break;      

      result = test_timer_create (mode, test_timer_accuracy);
      if (result != 0)
	break;

      result = test_timer_wait_all_done ();
      if (result != 0)
	break;

      result = test_timer_remove ();
      if (result != 0)
	break;

      mode = ~mode;
    }
  result = test_end ("timer-test");
  if (result == 0)
    {
      timer_benchs ();
    }
  return result;
}
